package sdk

import (
	"context"
	"net"
	"strings"
)

// IPinRanges returns a slice of all IP in all given IP ranges
// i.e 72.44.1.240/28,72.42.1.23/27
func IPinRanges(ctx context.Context, IPranges string) ([]string, error) {
	var ips []string
	ranges := strings.Split(IPranges, ",")
	for _, r := range ranges {
		i, err := IPinRange(ctx, r)
		if err != nil {
			return nil, err
		}
		ips = append(ips, i...)
	}
	return ips, nil
}

// IPinRange returns a slice of all IP in given IP range
// i.e 10.35.11.240/28
func IPinRange(ctx context.Context, IPrange string) ([]string, error) {
	ip, ipnet, err := net.ParseCIDR(IPrange)
	if err != nil {
		return nil, err
	}

	var ips []string
	for ip2 := ip.Mask(ipnet.Mask); ipnet.Contains(ip2); inc(ip2) {
		ips = append(ips, ip2.String())
	}

	return ips, nil
}

func inc(ip net.IP) {
	for j := len(ip) - 1; j >= 0; j-- {
		ip[j]++
		if ip[j] > 0 {
			break
		}
	}
}
